package PScm;

use strict;
use warnings;
use PScm::Read;
use PScm::Env;
use PScm::Primitive;
use PScm::SpecialForm;
use PScm::Class;
use PScm::Continuation;
use FileHandle;

require Exporter;

our @ISA    = qw(Exporter);
our @EXPORT = qw(ReadEvalPrint);

=head1 NAME

PScm - PScm interpreter written in Perl

=head1 SYNOPSIS

  use PScm;
  ReadEvalPrint($in_filehandle[, $out_filehandle]);

=head1 DESCRIPTION

Just messing about, A toy lisp interpreter.

=cut

my @thread_queue;

sub ReadEvalPrint {
    my ($infh, $outfh) = @_;

    $outfh ||= new FileHandle(">-");
    my $reader      = new PScm::Read($infh);
    my $initial_env;
    $initial_env = new PScm::Env(
        let          => new PScm::SpecialForm::Let(),
        '*'          => new PScm::Primitive::Multiply(),
        '-'          => new PScm::Primitive::Subtract(),
        '+'          => new PScm::Primitive::Add(),
        if           => new PScm::SpecialForm::If(),
        lambda       => new PScm::SpecialForm::Lambda(),
        list         => new PScm::Primitive::List(),
        car          => new PScm::Primitive::Car(),
        cdr          => new PScm::Primitive::Cdr(),
        cons         => new PScm::Primitive::Cons(),
        letrec       => new PScm::SpecialForm::LetRec(),
        'let*'       => new PScm::SpecialForm::LetStar(),
        eval         => new PScm::SpecialForm::Eval(),
        macro        => new PScm::SpecialForm::Macro(),
        quote        => new PScm::SpecialForm::Quote(),
        'set!'       => new PScm::SpecialForm::Set(),
        begin        => new PScm::SpecialForm::Begin(),
        define       => new PScm::SpecialForm::Define(),
        'make-class' => new PScm::SpecialForm::MakeClass(),
        'call/cc'    => new PScm::SpecialForm::CallCC(),
        print        => new PScm::SpecialForm::Print($outfh),
        spawn        => new PScm::SpecialForm::Spawn(),
        exit         => new PScm::SpecialForm::Exit(),
    );

    $initial_env->Define(
        PScm::Expr::Symbol->new("root"),
        PScm::Class::Root->new($initial_env)
    );
    __PACKAGE__->new_thread(cont { repl($initial_env, $reader, $outfh) });
    trampoline();
}

sub repl {
    my ($env, $reader, $outfh) = @_;
    $reader->Read(
        cont {
            my ($expr) = @_;
            $expr->Eval(
                $env,
                cont {
                    my ($result) = @_;
                    $result->Print(
                        $outfh,
                        cont {
                            repl($env, $reader, $outfh);
                        }
                    )
                }
            )
        }
    )
}

sub new_thread {
    my ($self, $cont) = @_;
    push @thread_queue, $cont;
}

sub trampoline {
    while (@thread_queue) {
        my $cont = shift @thread_queue;
        $cont = $cont->Bounce();
        push @thread_queue, $cont if defined $cont;
    }
}

sub Print {
    my ($expr, $outfh, $cont) = @_;
    print $outfh $expr->as_string, "\n";
    $cont->Cont($expr);
}

sub as_string { ref($_[0]); }

sub new { bless {}, $_[0] }

1;
