use strict;
use warnings;
use Test::More;
use lib 't/lib';
use PScm::Test tests => 2;

BEGIN { use_ok('PScm') }

eval_ok(<<EOF, <<EOR, 'yield');
(define yield
  (macro (value)
    '(call/cc
      (lambda (^here)
        (^return (list ^here ,value))))))

(define yielder
  (macro (body)
   '(let ((firsttime 1)
         (^resume 0)
         (^return 0))
     (lambda ()
       (if firsttime
         (let ((res (call/cc
                      (lambda (^cont)
                        (begin
                          (set! ^return ^cont)
                          ,body)))))
           (begin
               (set! firsttime 0)
               (set! ^resume (car res))
               (car (cdr res))))
         (let ((res (call/cc
                      (lambda (^cont)
                        (begin
                          (set! ^return ^cont)
                          (^resume))))))
           (begin
               (set! ^resume (car res))
               (car (cdr res)))))))))

(define while
  (macro (test body)
    '(letrec
      ((loop
         (lambda ()
           (if ,test
             (begin
               ,body
               (loop))
             ()))))
      (loop))))

(define fib
  (yielder
    (letrec ((fib-loop
               (lambda (i j)
                 (begin
                   (yield i)
                   (fib-loop j (+ i j))))))
      (fib-loop 0 1))))

(let ((n 10))
  (while n
    (begin
      (set! n (- n 1))
      (print (fib)))))
EOF
yield
yielder
while
fib
0
1
1
2
3
5
8
13
21
34
()
EOR

# vim: ft=perl
