use strict;
use warnings;
use Test::More;
use lib 't/lib';
use PScm::Test tests => 6;

BEGIN { use_ok('PScm') }


eval_ok(<<EOF, "account", 'classes');
(define account
    (make-class root (amount)
           (init (x) (set! amount x))
    ))
EOF

eval_ok(<<EOF, <<EOR, 'objects');
(define account
    (make-class
           root
           (balance)
           (init (x) (set! balance x))
           (balance () balance)
           (withdraw (x) (set! balance (- balance x)))
    ))
(define myaccount (account 10))
(myaccount balance)
(myaccount withdraw 2)
(myaccount balance)
EOF
account
myaccount
10
8
8
EOR

eval_ok(<<EOF, <<EOR, 'inheritance');
(define account
    (make-class
           root
           (balance)
           (init (x) (set! balance x))
           (balance () balance)
           (withdraw (x) (set! balance (- balance x)))
           (deposit (x) (set! balance (+ balance x)))
    ))
(define interest-account
    (make-class
           account
           (rate)
           (init (x r)
                (begin
                    (super init x)
                    (set! rate r)))
           (accumulate ()
                (this deposit (* (this balance) rate)))
    ))
(define myaccount (interest-account 10 2))
(myaccount balance)
(myaccount withdraw 2)
(myaccount balance)
(myaccount accumulate)
(myaccount balance)
EOF
account
interest-account
myaccount
10
8
8
24
24
EOR

eval_ok(<<EOF, <<EOR, 'class variables');
(define counter-class
    (let ((count 0))
        (make-class
            root
            ()
            (init () (set! count (+ count 1)))
            (count () count)
        )))
(define o1 (counter-class))
(o1 count)
(let ((o2 (counter-class))
      (o3 (counter-class)))
     (o1 count))
EOF
counter-class
o1
1
3
EOR

eval_ok(<<EOF, <<EOR, 'super calls');
(define c1
   (make-class
       root
       ()
       (ma () (this mb))
       (mb () 0)
   ))
(define c2
   (make-class
       c1
       ()
       (ma () (super ma))
       (mb () 1)
   ))
((c2) ma)
EOF
c1
c2
1
EOR

# vim: ft=perl
